package link;

import abstraction.CustomSet;
import abstraction.CustomSetPersistenceDecorator;
import abstraction.PersistenceAdapter;
import exeptions.LinkAlreadyExists;
import exeptions.LinkDoesNotExist;

import java.util.Optional;

public class LinkRepository {

	final private CustomSet<Link> links;

	public LinkRepository(PersistenceAdapter<Link> linkPersistenceAdapter) {
		this.links = new CustomSetPersistenceDecorator(linkPersistenceAdapter);
	}

	public Optional<Link> getById(LinkId id) {
		return links.stream().filter(link -> link.getId().equals(id)).findFirst();
	}

	public Optional<Link> getByUrl(LinkUrl url) {
		return links.stream().filter(link -> link.getUrl().equals(url)).findFirst();
	}

	public void removeById(LinkId id) {
		getById(id).ifPresentOrElse(this::remove, () -> {
			throw new LinkDoesNotExist("Tried removing a link with id " + id + " but it does not exist.");
		});
	}

	public void removeByUrl(LinkUrl url) {
		getByUrl(url).ifPresentOrElse(this::remove, () -> {
			throw new LinkDoesNotExist("Tried removing a link with url " + url + " but it does not exist.");
		});
	}

	public void add(Link link) {
		checkDuplicates(link);
		links.add(link);
	}

	private void remove(Link link) {
		links.remove(link);
	}

	private void checkDuplicates(Link newLink) {
		if (this.getById(newLink.getId()).isPresent()) {
			throw new LinkAlreadyExists("A link with the id " + newLink.getId() + " already exitsts");
		}

		if (this.getByUrl(newLink.getUrl()).isPresent()) {
			throw new LinkAlreadyExists("A link with the url " + newLink.getUrl() + " already exitsts");
		}
	}
}
